use std::io;

use anyhow::Error as AnyError;
use axum::{http, response::IntoResponse};
use hyper;
use reqwest;
use reqwest_middleware;
use thiserror::Error;

#[derive(Error, Debug)]
pub enum AppError {
    #[error("Axum Error")]
    AxumError(#[from] axum::Error),
    #[error("Axum Http Error")]
    AxumHttpError(#[from] http::Error),
    #[error("Reqwest Error")]
    ReqwestError(#[from] reqwest::Error),
    #[error("Reqwest Middleware Error")]
    ReqwestMiddlewareError(#[from] reqwest_middleware::Error),
    #[error("Client Error")]
    ClientError(#[from] hyper::Error),
    #[error("Io Error")]
    IoError(#[from] io::Error),
    #[error("Unknown error")]
    Unknown(#[from] AnyError),
}

impl IntoResponse for AppError {
    fn into_response(self) -> axum::response::Response {
        tracing::error!("::AppError::into_response: {:?}", self);
        (
            http::StatusCode::INTERNAL_SERVER_ERROR,
            format!("Something went wrong: {:?}", self),
        )
            .into_response()
    }
}
